#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_opengl.h"

#include "input.h"

#include "game.h"
#include "visuals.h"
#include "screens.h"
#include "interface.h"
#include "logic.h"

extern Game* game;
extern Visuals* visuals;
extern Screens* screens;
extern Interface* interface;
extern Logic* logic;

//-------------------------------------------------------------------------------------------------
Input::Input(void)
{
    KeyOnKeyboardPressedByUser = (char)NULL;
    MouseButtonPressed[0] = false;
    MouseButtonPressed[1] = false;

    if (SDL_NumJoysticks() == 0)
    {
        JoystickDeviceOne = NULL;
        JoystickDeviceTwo = NULL;
        fprintf(stdout, "No USB joysticks are plugged in\n");
    }
    else
    {
        if (SDL_NumJoysticks()>0)
        {
            JoystickDeviceOne = SDL_JoystickOpen(0);
            NumberOfJoyButtons[0] = SDL_JoystickNumButtons(JoystickDeviceOne);
            NumberOfJoyAxises[0] = SDL_JoystickNumAxes(JoystickDeviceOne);
            fprintf(stdout, "SDL Joystick 0 initialized\n");
        }

        if (SDL_NumJoysticks()>1)
        {
            JoystickDeviceTwo = SDL_JoystickOpen(1);
            NumberOfJoyButtons[1] = SDL_JoystickNumButtons(JoystickDeviceTwo);
            NumberOfJoyAxises[1] = SDL_JoystickNumAxes(JoystickDeviceTwo);
            fprintf(stdout, "SDL Joystick 1 initialized\n");
        }
    }

    for (int joy = 0; joy < 2; joy++)
    {
        JoyUP[joy] = Axis1;
        JoyDOWN[joy] = Axis1;
        JoyLEFT[joy] = Axis0;
        JoyRIGHT[joy] = Axis0;
        JoyButton1[joy] = Button0;
        JoyButton2[joy] = Button1;
    }

    JoystickSetupProcess = JoySetupNotStarted;

    for (int index = 0; index < 4; index++)
    {
        JoystickDirection[index] = CENTER;
        JoystickButtonOne[index] = OFF;
        JoystickButtonTwo[index] = OFF;
        JoystickButton1Pressed[index] = false;
        JoystickButton2Pressed[index] = false;
    }

    RotateSetupProcess = RotateSetupNotStarted;
    UserDefinedRotateCounterClockwise = '^';
    UserDefinedRotateClockwise = '^';

    DelayAllUserInput = -1;
}

//-------------------------------------------------------------------------------------------------
Input::~Input(void)
{
    if (JoystickDeviceOne != NULL)  SDL_JoystickClose(JoystickDeviceOne);
    if (JoystickDeviceTwo != NULL)  SDL_JoystickClose(JoystickDeviceTwo);
}

//-------------------------------------------------------------------------------------------------
Uint8 Input::QueryJoysticksForAction(Uint8 joy)
{
Uint8 returnValue = 0;

	if (DelayAllUserInput > 0)  return(0);

	if (joy == 0 && SDL_JoystickOpened(0))
	{
		SDL_JoystickUpdate();

        for (int index = 0; index < NumberOfJoyAxises[joy]; index++)
        {
            Sint16 joyAxis = 0;
            joyAxis = SDL_JoystickGetAxis(JoystickDeviceOne, index);
            if (joyAxis < -16383)
            {
                returnValue = Axis0+index;
                return(returnValue);
            }
            else if (joyAxis > 16383)
            {
                returnValue = Axis0+index;
                return(returnValue);
            }
        }

        for (int index = 0; index < NumberOfJoyButtons[joy]; index++)
        {
            if ( SDL_JoystickGetButton(JoystickDeviceOne, index) )
            {
                returnValue = Button0+index;
                return(returnValue);
            }
        }
    }

	if (joy == 1 && SDL_JoystickOpened(1))
	{
		SDL_JoystickUpdate();

        for (int index = 0; index < NumberOfJoyAxises[joy]; index++)
        {
            Sint16 joyAxis = 0;
            joyAxis = SDL_JoystickGetAxis(JoystickDeviceTwo, index);
            if (joyAxis < -16383)
            {
                returnValue = Axis0+index;
                return(returnValue);
            }
            else if (joyAxis > 16383)
            {
                returnValue = Axis0+index;
                return(returnValue);
            }
        }

        for (int index = 0; index < NumberOfJoyButtons[joy]; index++)
        {
            if ( SDL_JoystickGetButton(JoystickDeviceTwo, index) )
            {
                returnValue = Button0+index;
                return(returnValue);
            }
        }
    }

    return(returnValue);
}

//-------------------------------------------------------------------------------------------------
void Input::GetAllUserInput(void)
{
    KeyOnKeyboardPressedByUser = (char)NULL;
    MouseButtonPressed[0] = false;
    MouseButtonPressed[1] = false;

    for (int index = 0; index < 4; index++)
    {
        JoystickDirection[index] = CENTER;
        JoystickButtonOne[index] = OFF;
        JoystickButtonTwo[index] = OFF;
    }

    if (DelayAllUserInput > -1)
    {
        DelayAllUserInput--;
        return;
    }

    Uint8 *keystates = SDL_GetKeyState(NULL);

    bool shiftKeyPressed  = false;
    if ( (keystates[SDLK_LSHIFT])||(keystates[SDLK_RSHIFT]) ) shiftKeyPressed = true;

    while ( SDL_PollEvent(&Event) )
    {
        switch (Event.type)
        {
            case SDL_QUIT:
                KeyOnKeyboardPressedByUser = '~';
                break;

            case SDL_ACTIVEEVENT:
                switch(Event.active.gain)
                {
                    case 0:
                        logic->GameDisplayChanged = true;;
                        screens->ScreenMustBeDrawn = true;
                        break;

                    case 1:
                        logic->GameDisplayChanged = true;;
                        screens->ScreenMustBeDrawn = true;
                        break;

                    default:
                        break;
                }
                break;

            case SDL_VIDEORESIZE:
                visuals->UnloadAllTextsFromMemory();
                visuals->UnloadSpritesFromMemory();
                interface->UnloadGUIFromMemory();

                visuals->ScreenWidth = Event.resize.w;
                visuals->ScreenHeight = Event.resize.h;

                visuals->Screen = SDL_SetVideoMode(visuals->ScreenWidth, visuals->ScreenHeight, 0
                                                   , SDL_OPENGL | SDL_RESIZABLE);
                if (!visuals->Screen)
                {
                    printf("ERROR: SDL+OpenGL window resize failed\n");
                    game->CoreFailure = true;
                    return;
                }
                else fprintf(stdout, "SDL+OpenGL window resized\n");

                glMatrixMode(GL_PROJECTION);
                glLoadIdentity();
                glOrtho(0, 640, 480, 0, -1, 1);
                glMatrixMode(GL_MODELVIEW);
                glLoadIdentity();
                glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
                glDisable(GL_DEPTH_TEST);

                glViewport(0, 0, visuals->ScreenWidth, visuals->ScreenHeight);

                if ( interface->CreateAllGUIAndInitialize(false) == false )
                {
                    game->CoreFailure = true;
                    return;
                }

                if ( visuals->LoadSpritesIntoMemoryAndInitialize() == false )
                {
                    game->CoreFailure = true;
                    return;
                }

                if ( visuals->LoadAllTextsIntoMemory() == false )
                {
                    game->CoreFailure = true;
                    return;
                }

                screens->ScreenMustBeDrawn = true;
                break;

            case SDL_KEYDOWN:
                switch( Event.key.keysym.sym )
                {
                    case SDLK_ESCAPE:
                        if (screens->ScreenToDisplay == TitleScreen)
                        {
                            KeyOnKeyboardPressedByUser = '~';
                        }
                        else
                        {
                            screens->ScreenTransitionStatus = FadeOut;
                            screens->ExitToTitleScreen = true;
                        }

                        game->FrameLock = 17;
                        break;

                    case SDLK_SPACE:
                        KeyOnKeyboardPressedByUser = '_';
                        break;

                    case SDLK_RETURN:
                        KeyOnKeyboardPressedByUser = '/';
                        break;

                    case SDLK_BACKSPACE:
                        KeyOnKeyboardPressedByUser = '<';
                        break;

                    case SDLK_PLUS:
                        KeyOnKeyboardPressedByUser = '+';
                        break;

                    case SDLK_F12:
                        KeyOnKeyboardPressedByUser = '|';
                        break;

                    case SDLK_F1:
                        KeyOnKeyboardPressedByUser = '!';
                        break;

                    case SDLK_F2:
                        KeyOnKeyboardPressedByUser = '@';
                        break;

                    case SDLK_EQUALS:
                        if (shiftKeyPressed)  KeyOnKeyboardPressedByUser = '+';
                        else  KeyOnKeyboardPressedByUser = '=';
                        break;

                    case SDLK_KP_PLUS:
                        KeyOnKeyboardPressedByUser = '+';
                        break;

                    default:
                        int asciiCharCode = Event.key.keysym.sym;

                        if (Event.key.keysym.sym > 47 && Event.key.keysym.sym < 58)
                        {
                            KeyOnKeyboardPressedByUser = (char)asciiCharCode;
                        }
                        else if (Event.key.keysym.sym > 96 && Event.key.keysym.sym < 123)
                        {
                            if (shiftKeyPressed)  KeyOnKeyboardPressedByUser = ( (char)asciiCharCode - 32 );
                            else  KeyOnKeyboardPressedByUser = (char)asciiCharCode;
                        }
                        break;
                }
        }

    }

    if ( (KeyOnKeyboardPressedByUser == 'x' || KeyOnKeyboardPressedByUser == UserDefinedRotateClockwise)
        && screens->ScreenToDisplay != NameInputKeyboardScreen )
        JoystickButtonTwo[Keyboard] = ON;

    if ( (KeyOnKeyboardPressedByUser == 'z' || KeyOnKeyboardPressedByUser == UserDefinedRotateCounterClockwise)
        && screens->ScreenToDisplay != NameInputKeyboardScreen )
        JoystickButtonOne[Keyboard] = ON;

    if      ( keystates[ SDLK_UP ]    )  JoystickDirection[Keyboard] = UP;
    else if ( keystates[ SDLK_DOWN ]  )  JoystickDirection[Keyboard] = DOWN;
    else if ( keystates[ SDLK_LEFT ]  )  JoystickDirection[Keyboard] = LEFT;
    else if ( keystates[ SDLK_RIGHT ] )  JoystickDirection[Keyboard] = RIGHT;
//------------------------------------------------------------------------
    SDL_GetMouseState(&MouseX, &MouseY);

    float tempX, tempXtwo;
    float tempY, tempYtwo;;
    if (visuals->ScreenWidth != 640)
    {
        tempX = visuals->ScreenWidth;
        tempXtwo = tempX / 640;
        tempX = MouseX;
        MouseX = (int)(tempX / tempXtwo);
    }

    if (visuals->ScreenHeight != 480)
    {
        tempY = visuals->ScreenHeight;
        tempYtwo = tempY / 480;
        tempY = MouseY;
        MouseY = (int)(tempY / tempYtwo);
    }

    for (int index = 0; index < 2; index++)
    {
        if ( SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(index+1) )
        {
            if (MouseButtonWasClicked[index] == false)
            {
                MouseButtonPressed[index] = true;
                MouseButtonWasClicked[index] = true;
            }
        }
        else  MouseButtonWasClicked[index] = false;
    }
//------------------------------------------------------------------------
    if ( SDL_JoystickOpened(0) )
	{
		SDL_JoystickUpdate();

		Sint16 joystickXmovement = 0;
        if (JoyLEFT[0] > Axis7)
        {
            if ( SDL_JoystickGetButton(JoystickDeviceOne, JoyLEFT[0]-9) )  JoystickDirection[JoystickOne] = LEFT;
            else  if ( SDL_JoystickGetButton(JoystickDeviceOne, JoyRIGHT[0]-9) )  JoystickDirection[JoystickOne] = RIGHT;
        }
        else joystickXmovement = SDL_JoystickGetAxis(JoystickDeviceOne, JoyLEFT[0]-1);

		Sint16 joystickYmovement = 0;
        if (JoyUP[0] > Axis7)
        {
            if ( SDL_JoystickGetButton(JoystickDeviceOne, JoyUP[0]-9) )  JoystickDirection[JoystickOne] = UP;
            else  if ( SDL_JoystickGetButton(JoystickDeviceOne, JoyDOWN[0]-9) )  JoystickDirection[JoystickOne] = DOWN;
        }
        else joystickYmovement = SDL_JoystickGetAxis(JoystickDeviceOne, JoyUP[0]-1);

		if (joystickYmovement < -16383)
		{
            JoystickDirection[JoystickOne] = UP;
		}
		else if (joystickYmovement > 16383)
		{
            JoystickDirection[JoystickOne] = DOWN;
		}
		else if (joystickXmovement < -16383)
		{
            JoystickDirection[JoystickOne] = LEFT;
		}
		else if (joystickXmovement > 16383)
		{
            JoystickDirection[JoystickOne] = RIGHT;
		}

        if (JoyButton1[0] > Axis7)
        {
	    	if ( SDL_JoystickGetButton(JoystickDeviceOne, JoyButton1[0]-9) )  JoystickButtonOne[JoystickOne] = ON;
        }
        else
        {
            Sint16 padAsButton = SDL_JoystickGetAxis(JoystickDeviceOne, JoyButton1[0]-1);
            if (padAsButton < -16383)  JoystickButtonOne[JoystickOne] = ON;
        }

        if (JoyButton2[0] > Axis7)
        {
    		if ( SDL_JoystickGetButton(JoystickDeviceOne, JoyButton2[0]-9) )  JoystickButtonTwo[JoystickOne] = ON;
        }
        else
        {
            Sint16 padAsButton = SDL_JoystickGetAxis(JoystickDeviceOne, JoyButton2[0]-1);
            if (padAsButton < -16383)  JoystickButtonTwo[JoystickOne] = ON;
        }
    }

    if ( SDL_JoystickOpened(1) )
	{
		SDL_JoystickUpdate();

		Sint16 joystickXmovement = 0;
        if (JoyLEFT[1] > Axis7)
        {
            if ( SDL_JoystickGetButton(JoystickDeviceTwo, JoyLEFT[1]-9) )  JoystickDirection[JoystickTwo] = LEFT;
            else  if ( SDL_JoystickGetButton(JoystickDeviceTwo, JoyRIGHT[1]-9) )  JoystickDirection[JoystickTwo] = RIGHT;
        }
        else joystickXmovement = SDL_JoystickGetAxis(JoystickDeviceTwo, JoyLEFT[1]-1);

		Sint16 joystickYmovement = 0;
        if (JoyUP[1] > Axis7)
        {
            if ( SDL_JoystickGetButton(JoystickDeviceTwo, JoyUP[1]-9) )  JoystickDirection[JoystickTwo] = UP;
            else  if ( SDL_JoystickGetButton(JoystickDeviceTwo, JoyDOWN[1]-9) )  JoystickDirection[JoystickTwo] = DOWN;
        }
        else joystickYmovement = SDL_JoystickGetAxis(JoystickDeviceTwo, JoyUP[1]-1);

		if (joystickYmovement < -16383)
		{
            JoystickDirection[JoystickTwo] = UP;
		}
		else if (joystickYmovement > 16383)
		{
            JoystickDirection[JoystickTwo] = DOWN;
		}
		else if (joystickXmovement < -16383)
		{
            JoystickDirection[JoystickTwo] = LEFT;
		}
		else if (joystickXmovement > 16383)
		{
            JoystickDirection[JoystickTwo] = RIGHT;
		}

        if (JoyButton1[1] > Axis7)
        {
	    	if ( SDL_JoystickGetButton(JoystickDeviceTwo, JoyButton1[1]-9) )  JoystickButtonOne[JoystickTwo] = ON;
        }
        else
        {
            Sint16 padAsButton = SDL_JoystickGetAxis(JoystickDeviceTwo, JoyButton1[1]-1);
            if (padAsButton < -16383)  JoystickButtonOne[JoystickTwo] = ON;
        }

        if (JoyButton2[1] > Axis7)
        {
    		if ( SDL_JoystickGetButton(JoystickDeviceTwo, JoyButton2[1]-9) )  JoystickButtonTwo[JoystickTwo] = ON;
        }
        else
        {
            Sint16 padAsButton = SDL_JoystickGetAxis(JoystickDeviceTwo, JoyButton2[1]-1);
            if (padAsButton < -16383)  JoystickButtonTwo[JoystickTwo] = ON;
        }
    }
//------------------------------------------------------------------------
    JoystickDirection[Any] = CENTER;
    JoystickButtonOne[Any] = OFF;
    JoystickButtonTwo[Any] = OFF;

    for (int index = 0; index < 3; index++)
    {
        if (JoystickDirection[Any] == CENTER)  JoystickDirection[Any] = JoystickDirection[index];
        if (JoystickButtonOne[Any] == OFF)  JoystickButtonOne[Any] = JoystickButtonOne[index];
        if (JoystickButtonTwo[Any] == OFF)  JoystickButtonTwo[Any] = JoystickButtonTwo[index];
    }
}
//-------------------------------------------------------------------------------------------------
